* INBROWSE.PRG	: Driver for Integrated Browse, compatible with Tabs
* Author     				: Christophe BIEHLMANN - Christophe BIEHLMANN Consultant
* CIS ID     				: 100010,3412
* Address				: 71, rue Sevin Vincent
*						: 92210 Saint Cloud
*						: FRANCE
* Fax        				: (33) 1-46-02-53-63
*
*
* Copyright			: (c) Christophe BIEHLMANN Consultant
*
*    	This software is provided "as is" without
*    any warranty expressed or implied. In no event shall
*    Christophe Biehlmann, contributors, or distributors be liable
*    for any damages.
*
*    This software requires GENSCRNX 2.0
*    and 3D 2.0, two public domain programs created by
*    Ken Levy at the JPL in Pasadena, California.
*
* Version				: v1.1
* Date					: 15 - Nov - 94
*
* Description			: Provides lots of features to integrate browse(s) in screens
*
* Thanks to			: Ken R. Levy - Jet Propulsion Laboratory for Genscrnx and 3D
*						: Steven M. Black - Steven Black Consulting for Tabs
*  		     				: Jason Landry - Exchange Resources, Inc. for Brwx
*
* This software has been made PUBLIC DOMAIN by the author
*
**********************************************************************************************************************************************
*	SETUP SNIPPET DIRECTIVE REFERENCE
*
* 	*:SCXDRV5 = InBrowse ( **** Must be after Tabs **** )
*
*	Example :
* 		*:SCXDRV5 = Tabs
* 		*:SCXDRV5 = InBrowse
* 		*:SCXDRV5 = 3d
*
*	*:BROWSE3D [ON] / OFF						enable or disable automatic support of 3D effects for browse window
*	*:BROWSEGET3D [OFF] / ON				enable or disable automatic support of 3D effects for boxes with objects
*
*	COMMENT SNIPPET DIRECTIVE REFERENCE
*
* 	1 - Directive for the box with the browse
*
*  		*:BROWSEDB <expc1>	   				Database
*
* 			Options for the Browse ( default option in [ ] )
*
*  				*:BROWSENAME <expc1>		   				Window name for the browse
*  				*:BROWSETITLE <expc1>		   				Title for the browse
*  				*:BROWSEHEIGHT [NORMAL] / HALF / NONE	Height of the title bar
* 			 	*:BROWSEFONT  <expc1>,<expn1> 				Font and size for browse
* 			 	*:BROWSEFOR <expl>			   				Clause FOR of the Browse ( Multiple directives possible )
*			  	*:BROWSEWHEN <expl>       						Clause WHEN of the Browse ( Multiple directives possible )
*			  	*:BROWSEKEY <exp>      	 						Clause Key of the Browse
* 			 	*:BROWSECLAUSES <expc1>       				Adds more clauses ( Multiple directives possible )
*			 	*:BROWSEFIELDS <expc1>     					Field clause ( Multiple directive possible )
*			  	*:BROWSEREFRESH [OFF] / ON   				Adds a SHOW WINDOW ... REFRESH SAME statement to SHOW GETS
*  				*:BROWSEMENU [ON] / OFF        					Adds clause NOMENU in the browse command
*			  	*:BROWSEBAR [HV] / H / V / NONE				Show horizontal Bar ( H ) and/or vertical Bar ( V ) in the Browse
* 			 	*:BROWSEQUIT [ON] / OFF      					In a browse, close the screen by Esc, Ctrl+End, Ctrl+q, Ctrl+w
*
*		you can add directive for 3d options ( e.g. *:3D -4 BOX )
*
* 	2 - Directive for the box with objects ( 1 box minimun )
*
*  		*:BROWSEGET [<expc1>]       						Get window name
*
*		you can add directive for 3d options ( e.g. *:3D -4 BOX )
*
*
*  See the sample screens for more information.
*
*
**********************************************************************************************************************************************

#DEFINE C_ARRAY1		15
#DEFINE C_CMD			1
#DEFINE C_NAME		2
#DEFINE C_DB			3
#DEFINE C_TITLE		4
#DEFINE C_FONT		5
#DEFINE C_FOR			6
#DEFINE C_WHEN		7
#DEFINE C_KEY			8
#DEFINE C_FLDS		9
#DEFINE C_CLAUSES	10
#DEFINE C_MENU		11
#DEFINE C_REFRESH	12
#DEFINE C_BAR			13
#DEFINE C_HEIGHT		14
#DEFINE C_QUIT			15

#DEFINE C_ARRAY2		6
#DEFINE C_CMD			1
#DEFINE C_NAME		2
#DEFINE C_VPOS1		3
#DEFINE C_VPOS2		4
#DEFINE C_HPOS1		5
#DEFINE C_HPOS2		6

IF TYPE('m.gsxversion')#'C' .OR. LEFT(m.gsxversion,4)<'2.0'
	GOTO BOTTOM
	=warning('inBrowse driver requires GENSCNRX 2.0 or later')
	RETURN .F.
ENDIF

IF m.drv_no#5
	GOTO BOTTOM
	=warning('inBrowse driver requires *:SCXDRV5')
	RETURN .F.
ENDIF

PRIVATE mi, mTabDrv, mInBrowseDrv
FOR mi = 1to ALEN( a_scxdrv, 1 )
	IF "TABS" $ a_scxdrv[ mi, 5 ]
		mTabDrv = mi
	ENDIF
	IF "INBROWSE" $ a_scxdrv[ mi, 5 ]
		mInBrowseDrv = mi
	ENDIF
NEXT
IF mTabDrv > mInBrowseDrv
	GOTO BOTTOM
	=warning('inBrowse driver must be after TABS driver')
	RETURN .F.
ENDIF
RELEASE mi, mTabDrv, mInBrowseDrv


PRIVATE mOldAlias, mOldRec, mMaxBrowse, mNbCount, mOrigine
PRIVATE mwname, mwfont, mwfsize, mwfontclause, mscnh, mscnw, mscncolor
PRIVATE mNbBrowse, maBrowse, mBrowsePref, mVBar, mHBar
PRIVATE mNbGet, maGet
PRIVATE mi, msec1, msec2, mInWind, cString, nI, mEscProc, mCtrlTabProc, mPgDnProc, mPgUpProc
PRIVATE mTabDrv, m3dBrowse, m3dGet
PRIVATE mHpixel, mVpixel, m3D, mOffset

DO updthrm WITH "Running Browse Integrator ","[WINDOWS] "+PROGRAM(),0

IF objType = 1 AND TYPE( "ScxSetupId" ) = "U"
	PUBLIC ScxSetupId, PjxRecord, ScxModal
	mOldAlias = ALIAS()
	SELECT PjxData
	PjxRecord		= RECNO()
	ScxModal 		= PjxData.modal
	mMaxBrowse 	= 0
	ScxSetupId	= 250
	SCAN FOR TYPE == "s"
		USE ( ALLTRIM( PjxData.name ) ) ALIAS _fCount AGAIN IN 0
		SELECT COUNT(*) FROM _fCount ;
			WHERE platform = "WINDOWS" ;
			AND objType = 7 ;
			AND !EMPTY( wordfindc( "*:BROWSEDB", comment ) );
			INTO ARRAY mNbCount
		mMaxBrowse = mMaxBrowse + mNbCount[ 1 ]
		SELECT _fCount
		USE
		SELECT PjxData
		IF mNbCount[ 1 ] > 0
			ScxSetupId = MIN( ScxSetupId, ScrnOrder + 1 )
		ENDIF
	ENDSCAN
	SELECT PjxData
	GO PjxRecord
	SELECT ( mOldAlias )
ENDIF

mTabDrv = .F.
IF objType = 1
	IF  "*:TABLABEL" $ UPPER( setupcode )
		mTabDrv = .T.
	ENDIF
	m3dBrowse 	= IIF( "OFF" $ UPPER( wordFindC( "*:BROWSE3D", setupcode ) ), .F., .T. )
	m3dGet 		= IIF( "ON" $ UPPER( wordFindC( "*:BROWSEGET3D", setupcode ) ), .T., .F. )
	IF EMPTY(name)
		REPLACE name WITH LOWER(SYS(2015))
	ENDIF
	IF EMPTY(fontface)
		REPLACE fontface WITH "Arial",fontsize WITH 8
	ENDIF

	mwname			= name
	mwfont			= ALLTRIM( fontface )
	mwfsize			= fontsize
	mwfontclause		= " FONT '" + ALLTRIM( fontface ) + "'," + ALLTRIM( STR( fontsize ) )
	mscnh				= HEIGHT
	mscnw			= WIDTH
	mscncolor			= " COLOR RGB(,,,"+ALLTRIM(STR(fillred))+","+ALLTRIM(STR(fillgreen))+","+ALLTRIM(STR(fillblue))+")"
	mVpixel 			= 1.000 / FONTMETRIC( 1, mwfont , mwfsize )
	mHpixel 			= 1.000 / FONTMETRIC( 6, mwfont , mwfsize )

ENDIF

SET MESSAGE TO "Defining browse child windows"
mNbBrowse = 0
SCAN FOR objtype = 7 AND wordsearch("*:BROWSEDB ","comment") # CHR(0)

	IF m3dBrowse OR ATC( "*:3D" , comment ) > 0
		SCATTER MEMO TO maSave
		m3D 			=  UPPER( wordFindC("*:3D", comment ) )
		mOffset 		= IIF(  "RAISED" $ m3D, 1.000, 0.000 )
		m3D 			= ABS( VAL( m3D ) )
		m3D			= IIF( m3D = 0, 2, m3D )
		REPL 	VPOS 	WITH VPOS 	+ ( ( m3D + mOffset ) * mVpixel ),;
			HPOS 		WITH HPOS 	+ ( ( m3D + mOffset ) * mHpixel ),;
			HEIGHT 	WITH HEIGHT	- ( ( ( m3D * 2 ) + 4 ) * mVpixel ),;
			WIDTH 	WITH WIDTH 	- ( ( ( m3D * 2 ) + 4 ) * mHpixel )
	ENDIF

	mNbBrowse = mNbBrowse + 1
	DIMENSION maBrowse( mNbBrowse, C_ARRAY1 )
	maBrowse[ mNbBrowse, C_NAME ]			= wordFindC( "*:BROWSENAME",		comment )
	maBrowse[ mNbBrowse, C_DB ]			= wordFindC( "*:BROWSEDB",			comment )
	maBrowse[ mNbBrowse, C_TITLE ]			= wordFindC( "*:BROWSETITLE",		comment, .F., 1, [ "]			, [" ] )
	maBrowse[ mNbBrowse, C_FLDS ]			= wordFindC( "*:BROWSEFIELDS",		comment, .T., 1, [ FIELDS ]	, [ ], [,] )
	maBrowse[ mNbBrowse, C_FONT ]			= wordFindC( "*:BROWSEFONT",		comment, .F., 1, [ FONT ]		, [ ] )
	maBrowse[ mNbBrowse, C_KEY ]			= wordFindC( "*:BROWSEKEY",			comment, .F., 1, [ KEY ]		, [ ] )
	maBrowse[ mNbBrowse, C_FOR ]			= wordFindC( "*:BROWSEFOR",			comment, .T., 1, [ FOR ]		, [ ], [ ] )
	maBrowse[ mNbBrowse, C_WHEN ]		= wordFindC( "*:BROWSEWHEN",		comment, .T., 1, [ WHEN ]		, [ ], [ ] )
	maBrowse[ mNbBrowse, C_CLAUSES ]		= wordFindC( "*:BROWSECLAUSES",	comment, .T., 1, [ ]				, [ ], [ ] )
	maBrowse[ mNbBrowse, C_REFRESH ]	= wordFindC( "*:BROWSEREFRESH",	comment )
	maBrowse[ mNbBrowse, C_MENU ]			= wordFindC( "*:BROWSEMENU",		comment )
	maBrowse[ mNbBrowse, C_BAR ]			= wordFindC( "*:BROWSEBAR",			comment )
	maBrowse[ mNbBrowse, C_HEIGHT ]		= wordFindC( "*:BROWSEHEIGHT",		comment )
	maBrowse[ mNbBrowse, C_QUIT ]			= wordFindC( "*:BROWSEQUIT",			comment )

	IF EMPTY( maBrowse[ mNbBrowse, C_NAME ] )
		maBrowse[ mNbBrowse, C_NAME ] = LOWER( SYS(2015) )
	ENDIF
	IF EMPTY( maBrowse[ mNbBrowse, C_TITLE ] )
		maBrowse[ mNbBrowse, C_TITLE ] = [ "] + maBrowse[ mNbBrowse, C_DB ] + [" ]
	ENDIF
	maBrowse[ mNbBrowse, C_REFRESH ]	= IIF( UPPER( maBrowse[ mNbBrowse, C_REFRESH ] ) = "ON", .T., .F. )
	maBrowse[ mNbBrowse, C_MENU ]			= IIF( UPPER( maBrowse[ mNbBrowse, C_MENU ] ) = "OFF", " NOMENU ", " " )
	IF EMPTY( maBrowse[ mNbBrowse, C_BAR ] )
		maBrowse[ mNbBrowse, C_BAR ] = "HV"
	ENDIF
	mOrigine	= 0.000
	mVBar 	= IIF( "V" $ UPPER( maBrowse[ mNbBrowse, C_BAR ] ), SYSMETRIC( 5 ), 0.000 )
	mHBar 	= IIF( "H" $ UPPER( maBrowse[ mNbBrowse, C_BAR ] ), SYSMETRIC( 8 ), 0.000 )
	maBrowse[ mNbBrowse, C_QUIT ]	= IIF( UPPER( maBrowse[ mNbBrowse, C_QUIT ] ) = "OFF", .T., .F. )
	IF UPPER( maBrowse[ mNbBrowse, C_HEIGHT ] ) == "HALF"
		mHBar = mHBar + ( SYSMETRIC( 9 ) / 2 )
	ENDIF
	IF UPPER( maBrowse[ mNbBrowse, C_HEIGHT ] ) == "NONE"
		mOrigine 	= - SYSMETRIC( 9 )
	ENDIF
	mHBar 	= mVpixel 	* ( mOrigine + mHBar )
	mVBar 	= mHpixel 	* mVBar
	mOrigine 	= mVpixel 	* mOrigine

	mBrowsePref	= LOWER(SYS(2015))
	mOldAlias		= LOWER(SYS(2015))

	maBrowse[ mNbBrowse, C_CMD ]	= ;
		"DEFINE WINDOW " + maBrowse(mNbBrowse, C_NAME) ;
		+ " FROM " + objpos() ;
		+ " SIZE " + STR( HEIGHT, 7, 3) + "," + STR( WIDTH, 7, 3 ) ;
		+ " IN WINDOW " + mwname + mwfontclause + mscncolor + m.cr_lf

	maBrowse[ mNbBrowse, C_CMD ] = maBrowse[ mNbBrowse, C_CMD ] + ;
		"DEFINE WINDOW " + mBrowsePref + IIF( UPPER( maBrowse[ mNbBrowse, C_HEIGHT ] ) == "HALF", " HALF ", "" ) ;
		+ " FROM "	+ STR( mOrigine, 7, 3 ) + ", 0 ";
		+ " SIZE " 	+ STR( HEIGHT - mHBar, 7, 3 ) + "," + STR( WIDTH - mVBar, 7, 3 ) ;
		+ " IN WINDOW " + maBrowse[ mNbBrowse, C_NAME ] + " " + mwfontclause + mscncolor + m.cr_lf

	maBrowse[ mNbBrowse, C_CMD ]	= maBrowse[ mNbBrowse, C_CMD ] +;
		mOldAlias + " = ALIAS()"																		+ m.cr_lf +;
		"SELECT " + maBrowse[ mNbBrowse, C_DB ]												+ m.cr_lf +;
		"BROWSE WINDOW " + mBrowsePref + " IN " + maBrowse[ mNbBrowse, C_NAME ] 		+;
		maBrowse[ mNbBrowse, C_MENU ] + maBrowse[ mNbBrowse, C_FONT ]				+;
		" NOWAIT SAVE TITLE " + maBrowse[ mNbBrowse, C_TITLE ]

	maBrowse[ mNbBrowse, C_CMD ] = maBrowse[ mNbBrowse, C_CMD ] + maBrowse[ mNbBrowse, C_FLDS ]
	maBrowse[ mNbBrowse, C_CMD ] = maBrowse[ mNbBrowse, C_CMD ] + maBrowse[ mNbBrowse, C_KEY ]
	maBrowse[ mNbBrowse, C_CMD ] = maBrowse[ mNbBrowse, C_CMD ] + maBrowse[ mNbBrowse, C_FOR ]
	maBrowse[ mNbBrowse, C_CMD ] = maBrowse[ mNbBrowse, C_CMD ] + maBrowse[ mNbBrowse, C_WHEN ]
	maBrowse[ mNbBrowse, C_CMD ] = maBrowse[ mNbBrowse, C_CMD ] + maBrowse[ mNbBrowse, C_CLAUSES ] + m.cr_lf

	maBrowse[ mNbBrowse, C_CMD ] = maBrowse[ mNbBrowse, C_CMD ] 						+;
		"SHOW WINDOW " + maBrowse[ mNbBrowse, C_NAME ]									+ m.cr_lf +;
		"RELEASE WINDOW " + mBrowsePref														+ m.cr_lf +;
		"IF !EMPTY(" + mOldAlias + ")"																+ m.cr_lf +;
		"     SELECT (" + mOldAlias + ")"																+ m.cr_lf +;
		"ELSE"																							+ m.cr_lf +;
		"     SELECT 0"																				+ m.cr_lf +;
		"ENDIF" 																						+ m.cr_lf +;
		"RELEASE " + mOldAlias + "," + mBrowsePref												+ m.cr_lf

	IF m3dBrowse OR ATC( "*:3D" , comment ) > 0
		GATHER FROM maSave MEMO
	ELSE
		DELETE
	ENDIF

ENDSCAN

SET MESSAGE TO "Defining get child windows"
mNbGet 	= 0
SCAN FOR objtype = 7 AND wordsearch("*:BROWSEGET","comment") # CHR(0)

	IF m3dGet OR ATC( "*:3D" , comment ) > 0 OR ATC( "*:FRAME3D" , comment ) > 0 )
		SCATTER MEMO TO maSave
		m3D 			=  UPPER( wordFindC("*:3D", comment ) )
		mOffset 		= IIF(  "RAISED" $ m3D, 1.000, 0.000 )
		m3D 			= ABS( VAL( m3D ) )
		m3D			= IIF( m3D = 0, 2, m3D )
		REPL 	VPOS 	WITH VPOS 	+ ( ( m3D + mOffset ) * mVpixel ),;
			HPOS 		WITH HPOS 	+ ( ( m3D + mOffset ) * mHpixel ),;
			HEIGHT 	WITH HEIGHT	- ( ( ( m3D * 2 ) + 4 ) * mVpixel ),;
			WIDTH 	WITH WIDTH 	- ( ( ( m3D * 2 ) + 4 ) * mHpixel )
	ENDIF
	mNbGet = mNbGet + 1
	DIMENSION maGet( mNbGet, C_ARRAY2 )
	maGet[ mNbGet, C_NAME ]	= wordFindC("*:BROWSEGET", comment )
	maGet[ mNbGet, C_VPOS1 ]	= VPOS
	maGet[ mNbGet, C_VPOS2 ]	= VPOS + HEIGHT
	maGet[ mNbGet, C_HPOS1 ]	= HPOS
	maGet[ mNbGet, C_HPOS2 ]	= HPOS + WIDTH

	IF EMPTY( maGet[ mNbGet, C_NAME ] )
		maGet[ mNbGet, C_NAME ] = LOWER( SYS(2015) )
	ENDIF

	maGet[ mNbGet, C_CMD ] = ;
		"DEFINE WINDOW " + maGet[ mNbGet, C_NAME ] + " NONE IN WINDOW " + mwname ;
		+ " FROM "	+ STR( maGet[ mNbGet, C_VPOS1 ], 7, 3 ) + "," + STR( maGet[ mNbGet, C_HPOS1 ], 7, 3 ) ;
		+ " TO " 	+ STR( maGet[ mNbGet, C_VPOS2 ], 7, 3 ) + "," + STR( maGet[ mNbGet, C_HPOS2 ], 7, 3 ) ;
		+ " " + mwfontclause + mscncolor + m.cr_lf

	IF m3dGet OR ATC( "*:3D" , comment ) > 0 OR ATC( "*:FRAME3D" , comment ) > 0 )
		GATHER FROM maSave MEMO
	ELSE
		DELETE
	ENDIF

ENDSCAN

SET MESSAGE TO "Adding supporting procedures and functions"
SCAN FOR objtype = 1
	IF m.scxcount = ScxSetupId

		mEscProc		= LOWER(SYS(2015))
		mCtrlTabProc	= LOWER(SYS(2015))
		mPgDnProc	= LOWER(SYS(2015))
		mPgUpProc	= LOWER(SYS(2015))

		REPLACE proccode WITH ;
			"ON KEY LABEL ESC			&_aBrowseKey[ 1 ]"				+ m.cr_lf +;
			"ON KEY LABEL Ctrl+w 		&_aBrowseKey[ 2 ]"				+ m.cr_lf +;
			"ON KEY LABEL Ctrl+End 	&_aBrowseKey[ 3 ]"				+ m.cr_lf +;
			"ON KEY LABEL Ctrl+q 		&_aBrowseKey[ 4 ]"				+ m.cr_lf +;
			"ON KEY LABEL Ctrl+Tab 	&_aBrowseKey[ 5 ]"				+ m.cr_lf +;
			"ON KEY LABEL PGDN 		&_aBrowseKey[ 6 ]"				+ m.cr_lf +;
			"ON KEY LABEL PGUP 		&_aBrowseKey[ 7 ]"				+ m.cr_lf +;
			m.cr_lf +;
			proccode															+ m.cr_lf +;
			"PROCEDURE " + mEscProc										+ m.cr_lf +;
			"PUSH KEY CLEAR"												+ m.cr_lf +;
			"IF ASCAN( _aBrowseQuit, WONTOP() ) = 0"					+ m.cr_lf +;
			"	ACTIVATE WINDOW " + maGet[ 1, C_NAME ]				+ m.cr_lf +;
			"	CLEAR READ"												+ m.cr_lf +;
			"ENDIF"															+ m.cr_lf +;
			"POP KEY"															+ m.cr_lf +;
			"RETURN"															+ m.cr_lf

		REPLACE proccode WITH ;
			proccode															+ m.cr_lf +;
			"PROCEDURE " + mCtrlTabProc									+ m.cr_lf +;
			"PRIVATE cParent, cWind, cwind1, nTmp, cGet, cTop"			+ m.cr_lf +;
			"PUSH KEY CLEAR"												+ m.cr_lf +;
			"nTmp 	= 0"		 												+ m.cr_lf +;
			"cTop	= WONTOP()"												+ m.cr_lf +;
			"cGet 	= cTop"													+ m.cr_lf +;
			"cParent	= WPARENT( cTop )"									+ m.cr_lf +;
			"IF ASCAN( _aBrowse, cTop ) > 0"								+ m.cr_lf +;
			"	cParent = WPARENT( cParent )"								+ m.cr_lf +;
			"ENDIF"															+ m.cr_lf +;
			"nb = WCHILD( cParent )"											+ m.cr_lf +;
			"  cWind = WCHILD( cParent, 0 )"									+ m.cr_lf +;
			"FOR mi = 1 to nb"												+ m.cr_lf +;
			"	cWind1 = WCHILD( cWind, 0) "								+ m.cr_lf +;
			"	IF !EMPTY( cWind1 ) AND ASCAN( _aBrowse, cWind1 ) > 0"+ m.cr_lf +;
			"		nTmp = nTmp + 1"		 								+ m.cr_lf +;
			"		DIMENSION aTmp( nTmp )"		 						+ m.cr_lf +;
			"		aTmp[ nTmp ] = cWind1 "									+ m.cr_lf +;
			"	ELSE"															+ m.cr_lf +;
			"		IF !EMPTY(  cWind ) "										+ m.cr_lf +;
			"			cGet =  cWind"							 				+ m.cr_lf +;
			"		ENDIF"													+ m.cr_lf +;
			"	ENDIF"														+ m.cr_lf +;
			"	cWind = WCHILD( cParent, 1) "								+ m.cr_lf +;
			"NEXT"																+ m.cr_lf

		REPLACE proccode WITH ;
			proccode															+ m.cr_lf +;
			"IF nTmp > 0"														+ m.cr_lf +;
			"	=ASORT( aTmp )"												+ m.cr_lf +;
			"	mi = ASCAN( aTmp, cTop )"									+ m.cr_lf +;
			"	IF mi < ALEN( aTmp )"		 								+ m.cr_lf +;
			"		ACTIVATE WINDOW  ( aTmp[ mi + 1 ] ) "					+ m.cr_lf +;
			"	ELSE"															+ m.cr_lf +;
			"		ACTIVATE WINDOW ( cGet ) "								+ m.cr_lf +;
			"	ENDIF"														+ m.cr_lf +;
			"ENDIF"															+ m.cr_lf +;
			"POP KEY"															+ m.cr_lf +;
			"RETURN"															+ m.cr_lf

		IF mTabDrv
			REPLACE proccode WITH ;
				proccode																	+ m.cr_lf +;
				"PROCEDURE " + mPgUpProc											+ m.cr_lf +;
				"PRIVATE cVar, nObj, cTmp, cCar, nI"									+ m.cr_lf +;
				"PUSH KEY CLEAR"														+ m.cr_lf +;
				"FOR nObj = _Curobj TO 1 STEP -1 "										+ m.cr_lf +;
				"	cVar = UPPER( OBJVAR( nObj ) )"									+ m.cr_lf +;
				"	IF [___TABS] $ cVar "													+ m.cr_lf +;
				"		cCar = CHR( ASC( SUBST( cVar, 10, 1 ) ) - 1 )"					+ m.cr_lf +;
				"		cTmp = [___TABS] + REPLICATE( cCar, 2 )"						+ m.cr_lf +;
				"		IF TYPE( cTmp ) # [U] "											+ m.cr_lf +;
				"			_CurObj = OBJNUM( &cTmp ) "								+ m.cr_lf +;
				"			POP KEY"														+ m.cr_lf +;
				"			RETURN"														+ m.cr_lf +;
				"		ELSE"																+ m.cr_lf

			REPLACE proccode WITH ;
				proccode																	+ m.cr_lf +;
				"			FOR nI = 65 TO 90 "											+ m.cr_lf +;
				"				cTmp = [___TABS] + REPLICATE( CHR( nI ) , 2 )"		+ m.cr_lf +;
				"				IF TYPE( cTmp ) = [U] "									+ m.cr_lf +;
				"					cTmp = [___TABS] + REPLICATE( CHR( nI - 1) , 2 )"	+ m.cr_lf +;
				"					_CurObj = OBJNUM( &cTmp ) "						+ m.cr_lf +;
				"					POP KEY"												+ m.cr_lf +;
				"					RETURN"												+ m.cr_lf +;
				"				ENDIF"													+ m.cr_lf +;
				"			NEXT"															+ m.cr_lf +;
				"		ENDIF"															+ m.cr_lf +;
				"	ENDIF"																+ m.cr_lf +;
				"NEXT"																		+ m.cr_lf +;
				"_CurObj = OBJNUM( ___TabsAA ) "										+ m.cr_lf +;
				"POP KEY"																	+ m.cr_lf +;
				"RETURN"																	+ m.cr_lf

			REPLACE proccode WITH ;
				proccode																	+ m.cr_lf +;
				"PROCEDURE " + mPgDnProc											+ m.cr_lf +;
				"PRIVATE cVar, nObj, cTmp, cCar, nI"									+ m.cr_lf +;
				"PUSH KEY CLEAR"														+ m.cr_lf +;
				"FOR nObj = _Curobj TO 1 STEP -1 "										+ m.cr_lf +;
				"	cVar = UPPER( OBJVAR( nObj ) )"									+ m.cr_lf +;
				"	IF [___TABS] $ cVar "													+ m.cr_lf +;
				"		cCar = CHR( ASC( SUBST( cVar, 10, 1 ) ) + 1 )"					+ m.cr_lf +;
				"		cTmp = [___TABS] + REPLICATE( cCar, 2 )"						+ m.cr_lf +;
				"		IF TYPE( cTmp ) # [U] "											+ m.cr_lf +;
				"			_CurObj = OBJNUM( &cTmp ) "								+ m.cr_lf +;
				"			POP KEY"														+ m.cr_lf +;
				"			RETURN"														+ m.cr_lf +;
				"		ELSE"																+ m.cr_lf +;
				"			_CurObj = OBJNUM( ___TabsAA ) "							+ m.cr_lf +;
				"			POP KEY"														+ m.cr_lf +;
				"			RETURN"														+ m.cr_lf +;
				"		ENDIF"															+ m.cr_lf +;
				"	ENDIF"																+ m.cr_lf +;
				"NEXT"																		+ m.cr_lf

			REPLACE proccode WITH ;
				proccode																	+ m.cr_lf +;
				"_CurObj = OBJNUM( ___TabsAA ) "										+ m.cr_lf +;
				"POP KEY"																	+ m.cr_lf +;
				"RETURN"																	+ m.cr_lf

			cString = ALLTRIM( DEACTIVATE )
			IF deacttype = 0
				cString = "RETURN " + IIF( !EMPTY( cString  ), cString, ".F." )
			ELSE
				cString = cString + m.cr_lf + "RETURN .F."
			ENDIF
			REPLACE deacttype WITH 1, DEACTIVATE WITH ;
				"ON KEY LABEL PgDn" 											+ m.cr_lf +;
				"ON KEY LABEL PgUp" 											+ m.cr_lf +;
				cString 															+ m.cr_lf

			cString = ALLTRIM( ACTIVATE )
			IF activtype = 0
				cString = "RETURN " + cString
			ELSE
				cString = cString + m.cr_lf + "RETURN"
			ENDIF
			REPLACE activtype WITH 1, ACTIVATE WITH ;
				"ON KEY LABEL PgDn DO " + mPgDnProc						+ m.cr_lf +;
				"ON KEY LABEL PgUp DO " + mPgUpProc						+ m.cr_lf +;
				cString 															+ m.cr_lf
		ENDIF

		msec1 = "#SECTION 1" 												+ m.cr_lf +;
			"DIMENSION _aBrowse[ " + STR( mMaxBrowse, 3 ) +" ]"		+ m.cr_lf +;
			"DIMENSION _aBrowseQuit[ " + STR( mMaxBrowse, 3 ) +" ]"	+ m.cr_lf +;
			"DIMENSION _aBrowseKey[ 7 ]"									+ m.cr_lf +;
			"_aBrowseQuit = [ ]"												+ m.cr_lf +;
			"_aBrowseKey = [ ]"												+ m.cr_lf +;
			"_aBrowse = [ ]"													+ m.cr_lf

		msec1 = msec1 														+;
			"_aBrowseKey[ 1 ]  = ON( 'KEY', 'ESC' )"							+ m.cr_lf +;
			"_aBrowseKey[ 2 ]  = ON( 'KEY', 'CTRL+W' )"						+ m.cr_lf +;
			"_aBrowseKey[ 3 ]  = ON( 'KEY', 'CTRL+END' )"					+ m.cr_lf +;
			"_aBrowseKey[ 4 ]  = ON( 'KEY', 'CTRL+Q' )"						+ m.cr_lf +;
			"_aBrowseKey[ 5 ]  = ON( 'KEY', 'CTRL+TAB' )"					+ m.cr_lf +;
			"_aBrowseKey[ 6 ]  = ON( 'KEY', 'PGDN' )"						+ m.cr_lf +;
			"_aBrowseKey[ 7 ]  = ON( 'KEY', 'PGUP' )"						+ m.cr_lf

		msec2 = "#SECTION 2"												+ m.cr_lf +;
			"ON KEY LABEL ESC 			DO " + mEscProc					+ m.cr_lf +;
			"ON KEY LABEL Ctrl+w 		DO " + mEscProc					+ m.cr_lf +;
			"ON KEY LABEL Ctrl+end 	DO " + mEscProc					+ m.cr_lf +;
			"ON KEY LABEL Ctrl+q 		DO " + mEscProc					+ m.cr_lf +;
			"ON KEY LABEL Ctrl+Tab 	DO " + mCtrlTabProc				+ m.cr_lf
	ELSE
		msec1 = "#SECTION 1" + m.cr_lf
		msec2 = "#SECTION 2" + m.cr_lf
	ENDIF

	FOR mi = 1 TO mNbBrowse
		msec1 = msec1 + "=AINS( _aBrowse, 1 )" + m.cr_lf  + "_aBrowse[ 1 ] = " + maBrowse[ mi, C_TITLE ] + m.cr_lf
		IF maBrowse[ mi, C_QUIT ]
			msec1 = msec1 + "=AINS( _aBrowseQuit, 1 )" + m.cr_lf  + "_aBrowseQuit[ 1 ] = " + maBrowse[ mi, C_TITLE ] + m.cr_lf
		ENDIF
		msec2 = msec2 + maBrowse[ mi, C_CMD ]
		IF maBrowse[ mi, C_REFRESH ]
			REPLACE SHOW WITH "SHOW WINDOW " + maBrowse[ mi, C_TITLE ] + " REFRESH SAME" + m.cr_lf + SHOW
		ENDIF
	NEXT

	FOR mi  = 1 TO mNbGet
		msec2 = msec2 + maGet[ mi, C_CMD ]
	NEXT

	DO CASE
		CASE "#SECTION 1" $ UPPER(setupcode) AND "#SECTION 2" $ UPPER(setupcode)
			REPLACE setupcode WITH strtranc( setupcode, "#SECTION 1", msec1 )
			REPLACE setupcode WITH strtranc( setupcode, "#SECTION 2", msec2 )
		CASE "#SECTION 1" $ UPPER(setupcode)
			REPLACE setupcode WITH strtranc( setupcode, "#SECTION 1", msec1 )
			REPLACE setupcode WITH setupcode + m.cr_lf + msec2
		CASE "#SECTION 2" $ UPPER(setupcode)
			REPLACE setupcode WITH setupcode + m.cr_lf + msec1
			REPLACE setupcode WITH strtranc( setupcode, "#SECTION 2", msec2 )
		OTHERWISE
			REPLACE setupcode WITH msec1 + m.cr_lf + setupcode + msec2
	ENDCASE

ENDSCAN

SET MESSAGE TO "Adjusting object positions within child window"
IF mTabDrv
	msec2 = ""
	SCAN FOR ObjType = 20  AND "___TABS" $ UPPER( name )
		mInWind = .F.
		FOR mi = 1 TO mNbGet
			IF BETWEEN( VPOS, maGet[mi, C_VPOS1], maGet[mi, C_VPOS2] ) ;
					AND BETWEEN(HPOS, maGet[mi, C_HPOS1], maGet[mi, C_HPOS2] )
				mInWind = .T.
				EXIT
			ENDIF
		NEXT

		IF !( mInWind )
			mNbGet = mNbGet + 1
			DIMENSION maGet( mNbGet, C_ARRAY2 )
			maGet[ mNbGet, C_NAME ]	= LOWER( SYS( 2015 ) )
			maGet[ mNbGet, C_VPOS1 ]	= VPOS
			maGet[ mNbGet, C_VPOS2 ]	= VPOS + HEIGHT
			maGet[ mNbGet, C_HPOS1 ]	= HPOS
			maGet[ mNbGet, C_HPOS2 ]	= HPOS + WIDTH
			msec2 =  msec2 + ;
				"DEFINE WINDOW " + maGet[ mNbGet, C_NAME ] + " NONE IN WINDOW " + mwname ;
				+ " FROM "	+ STR( maGet[ mNbGet, C_VPOS1 ], 7, 3 ) + "," + STR( maGet[ mNbGet, C_HPOS1 ], 7, 3 ) ;
				+ " TO " 	+ STR( maGet[ mNbGet, C_VPOS2 ], 7, 3 ) + "," + STR( maGet[ mNbGet, C_HPOS2 ], 7, 3 ) ;
				+ " " + mwfontclause + mscncolor + m.cr_lf
		ENDIF

	ENDSCAN
ENDIF

SCAN FOR ObjType > 4
	mInWind = .F.
	FOR mi = 1 TO mNbGet
		IF BETWEEN( VPOS, maGet[mi, C_VPOS1], maGet[mi, C_VPOS2] ) ;
				AND BETWEEN(HPOS, maGet[mi, C_HPOS1], maGet[mi, C_HPOS2] )
			REPL 	VPOS 	WITH ( VPOS - maGet[mi, C_VPOS1] ), ;
				HPOS 		WITH ( HPOS - maGet[mi, C_HPOS1] ), ;
				HEIGHT 	WITH MIN( HEIGHT, maGet[mi, C_VPOS2] - maGet[mi, C_VPOS1] - VPOS), ;
				WIDTH 	WITH MIN( WIDTH, maGet[mi, C_HPOS2] - maGet[mi, C_HPOS1] - HPOS)
			= INSTXT2("ACTIVATE WINDOW " + maGet[mi, C_NAME] + " SAME", -1 )
			mInWind = .T.
			EXIT
		ENDIF
	NEXT

	IF !( mInWind )
		= INSTXT2("ACTIVATE WINDOW " + mwname + " SAME", -1 )
	ENDIF

ENDSCAN

SCAN FOR ObjType = 1
	IF !EMPTY( msec2 )
		msec2 = "#SECTION 2" + m.cr_lf + msec2
		REPLACE setupcode WITH strtranc( setupcode, "#SECTION 2", msec2 )
	ENDIF
ENDSCAN

DO updthrm WITH "Running Browse Integrator","[WINDOWS] "+PROGRAM(),100
GO BOTTOM

IF ScxModal
	mAssocWind	= ""
	mOldAlias 	= ALIAS()
	SELECT PjxData
	mOldRec = RECNO()
	GO PjxRecord
	FOR mi = 1 TO mNbBrowse
		IF !EMPTY( maBrowse[ mi, C_TITLE ] )
			mAssocWind = mAssocWind + "," + maBrowse[ mi, C_TITLE ]
		ENDIF
	NEXT
	IF !EMPTY( mAssocWind )
		IF !EMPTY( PjxData.assocWinds )
			mAssocWind = "," + mAssocWind
		ENDIF
		REPL PjxData.assocWinds WITH PjxData.assocWinds + mAssocWind
	ENDIF
	SELECT PjxData
	GO mOldRec
	SELECT ( mOldAlias )
ENDIF

IF m.scxcount == m.screens
	RELEASE ScxSetupId, PjxRecord, ScxModal
ENDIF

FUNCTION WordFindC
*
* Author		: Christophe Biehlmann
* Created		: 12 Nov 94
* Parameters	: c_String 		= String to search
*				: u_Var			= String to search in
*				: l_Concat		= Logic	value for concatenation
*				: n_Occur		= Search from <n_Occur>th occurrence of c_String
*				: c_Before		= String to add before the result
*				: c_After		= String to add after the result
*				: c_Separator	= Separator to add after each occurrence
* Return			: String
*
PARAMETER 	c_String, c_Var, l_Concat, n_Occur, c_Before, c_After, c_Separator
PRIVATE 		n_MemoWidth, n_Pos, c_Return, c_Line, n_Len, n_LenSep

IF TYPE( "c_String" ) $ "MC"
	c_String = ALLTRIM( c_String )
ELSE
	c_String = ""
ENDIF
IF !TYPE( "c_Var" ) $ "MC"
	c_Var = ""
ENDIF
IF TYPE( "l_Concat" ) # "L"
	l_Concat = .F.
ENDIF
IF TYPE( "n_Occur" ) # "N"
	n_Occur = 1
ELSE
	n_Occur = MAX( n_Occur, 1 )
ENDIF
IF !TYPE( "c_Before" ) $ "MC"
	c_Before = ""
ENDIF
IF !TYPE( "c_After" ) $ "MC"
	c_After = ""
ENDIF
IF !TYPE( "c_Separator" ) $ "C"
	c_Separator = ""
ENDIF

n_MemoWidth = SET( 'memowidth' )
SET MEMOWIDTH TO 256

c_Return 	= ""
n_Len		= LEN( ALLTRIM( c_String ) )
n_LenSep	= LEN( ALLTRIM( c_Separator ) )
c_Var		= SUBST( c_Var, ATC( c_String, c_Var, n_Occur ) )
DO WHILE .T.
	n_Pos 	= ATCLINE( c_String, c_Var )
	IF n_Pos # 0
		c_Line = MLINE( c_Var, n_Pos )
		IF LEN( c_Line ) > n_Len
			c_Return = c_Return + ALLTRIM( SUBST( c_Line, n_Len + 1 ) )
		ENDIF
	ELSE
		EXIT
	ENDIF
	IF l_Concat
		c_Var = SUBST( c_Var, ATC( c_String, c_Var, 2 ) )
		IF !EMPTY( c_Var ) AND RIGHT( c_Return, n_LenSep ) # c_Separator
			c_Return = c_Return + c_Separator
		ENDIF
	ELSE
		EXIT
	ENDIF
ENDDO

IF RIGHT( c_Return, n_LenSep ) == c_Separator
	c_Return = LEFT( c_Return, LEN( c_Return ) - n_LenSep )
ENDIF
IF !EMPTY( c_Return )
	c_Return = c_Before + c_Return + c_After
ENDIF

SET MEMOWIDTH TO n_MemoWidth
RETURN c_Return


